
  { ExtraHelp v1.00 - Jon Webb, july 7th 1996                       }
  { Freeware; please report bugs, bug fixes, comments, enhancements }
  { etc. to j.webb@marketresponse.nl                                }

  { Neither the author nor his employer can be held responsible for }
  { any damage caused by the use of this utility                    }

  { This is the main unit; it defines the client which is used by   }
  { Delphi's VCS interface                                          }

unit Ehclient;

interface

uses
  Dialogs, VcsIntf, ToolIntf, SysUtils, Classes;

type
  TEHClient = class(TIVCSClient)
      constructor Create(ADelphi: TIToolServices);
      function GetIDString: string; override;
      procedure ExecuteVerb(Index: Integer); override;
      function GetMenuName: string; override;
      function GetVerb(Index: Integer): string; override;
      function GetVerbCount: Integer; override;
      function GetVerbState(Index: Integer): Word; override;
      procedure ProjectChange; override;
    private
      Delphi: TIToolServices;
      MenuItems: TStringList;
      HdxPath: String;
      SearchPaths: TStringList;
      AdditionalHelpFiles: TStringList;
      procedure HandleException(E: Exception);
      procedure GetHelpFiles(list: TStringList);
      procedure OpenHelpFile(helpFile: String);
      procedure About;
      procedure Options;
    protected
    public
  end;

implementation

uses virtintf, winprocs, wintypes,
     about, options, extraIni;

{THelpFile is used to store the list of help files}

type THelpFile = class(TObject)

       Title: string;                {Name to show user}
       Path: string;                 {Path to help file}
       available: boolean;           {True if help file was found}

       constructor Create(APath: string; SearchPaths: TStringList);

     end;

{-------------------- GLOBAL FUNCTIONS --------------------------}

{ Function to get environment variable without using WinDos unit }
{ - from Lloyds code snippets                                    }

function GetEnvStr( VarName : string ) : string;
var
  Len     : Word;
  EnvStz  : PChar;
  NameStz : array[ 0..180 ] of Char;
begin
  StrPCopy( NameStz, VarName );     { Covert VarName to PChar }
  Len := StrLen( NameStz );
  EnvStz := GetDosEnvironment;      { EnvStz holds entire env }

  while EnvStz^ <> #0 do
  begin                  { Pick off Variable Name and Compare }
    if ( StrLIComp( EnvStz, NameStz, Len ) = 0 ) and
       ( EnvStz[ Len ] = '=' ) then
    begin          { Convert to Pascal string before returing }
      Result := StrPas( EnvStz + Len + 1 );
      Exit;
    end;
    Inc( EnvStz, StrLen( EnvStz ) + 1 );   { Jump to Next Var }
  end;
  Result := '';
end;

{ Function to locate a file; checks actual file name first, then }
{ looks in the current directory and finally on the DOS path     }

function FindFileOnPath(var filename: string) : boolean;
  var f: string;
      path: string;
begin

  if FileExists(filename) then         {if filename is FQN}
  begin
    result := true;
    filename := ExpandFileName(filename);
  end
  else                                 {Not found - look in path}
  begin
    GetDir(0, path);
    path := path + ';' + GetEnvStr('PATH');

    f := FileSearch(filename, GetEnvStr('PATH'));
    result := (f<>'');
    if result then
      filename := f;
  end;

end;

{ Returns TRUE if filename is a help file }

function IsHelpFile(filename: string) : boolean;
begin
  result := upperCase(copy(filename,length(filename)-4, 4)) = '.HLP';
end;

{----------------------- THelpFile METHODS --------------------------}

constructor THelpFile.Create(APath: string; SearchPaths: TStringList);
  var i: integer;
      p: string;
begin
  inherited Create;

  {Try to find file in additional search paths}

  path := ExpandFileName(APath);
  if not FileExists(path) then
  begin

    path := ExtractFileName(path);
    p := path;
    i := 0;
    while (not FileExists(p)) and (i<searchPaths.Count) do
    begin
      p := SearchPaths[i];
      if p[length(p)]<>'\' then
        p := p + '\';
      p := p + path;
      inc(i);
    end;
    path := p;

    {Still not found - try DOS path}

    if not FileExists(path) then
      FindFileOnPath(Path);

  end;

  {To disable menu item...}

  available := FileExists(path);

  {Title for user display}

  title := ExtractFileName(path);
  title := lowerCase(copy(title, 1, length(title)-4)) + ' help';
  title[1] := UpCase(title[1]);

end;

{----------------------- TEHClient METHODS --------------------------}

{ For some reason or other this is never called when an exception    }
{ occurs. Any hints would be more than welcome. Currently an         }
{ exception in ExtraHelp causes Delphi to crash!                     }

procedure TEHClient.HandleException(E: Exception);
begin
  MessageDlg('ExtraHelp has detected an exception: '+
      e.message,
      mtInformation,
      [mbOk], 0);
end;

{ Constructor - initializes list of menu items by calling GetHelpFiles }

constructor TEHClient.Create(ADelphi: TIToolServices);
  var i: integer;
      helpFile: THelpFile;
begin
  TObject.Create;
  Delphi := ADelphi;

  try

    {Create objects to hold INI file settings}

    SearchPaths := TStringList.Create;
    AdditionalHelpFiles := TStringList.Create;

    {Read INI file}

    ReadIniFile(HDXPath, SearchPaths, AdditionalHelpFiles);

    {Create menu items}

    MenuItems := TStringList.Create;

    {Get help files from HDX}

    GetHelpFiles(MenuItems);

    {Add additional help files}

    if AdditionalHelpFiles.Count>0 then
    begin
      MenuItems.AddObject('', nil);
      for i := 0 to AdditionalHelpFiles.Count-1 do
      begin
        HelpFile := THelpFile.Create(AdditionalHelpFiles[i], SearchPaths);
        MenuItems.addObject(helpFile.Title, HelpFile);
      end;
    end;

    {Add option & about items}

    MenuItems.AddObject('', nil);
    MenuItems.AddObject('Options...', nil);
    MenuItems.AddObject('About ExtraHelp...', nil);

  except
    on E: Exception do
      HandleException(E);
  end;
end;

{ Searches for and reads DELPHI.HDX to create a list of help files }
{ Uses INI file setting first, then searches the current directory }
{ and the path to find HDX file. The name of the help file is      }
{ put in the string part of the string list; The THelpFile object  }
{ is in the object part                                            }

procedure TEHClient.GetHelpFiles(list: TStringList);
  var hdx: file of char;
      c: char;
      s: string;
      i,j,count: byte;
      path, msg: string;
      HelpFile : THelpFile;
begin

  if not FileExists(HdxPath) then
    MessageDlg('ExtraHelp: Unable to locate DELPHI.HDX. Please specify'#13#10+'using Options.',
       mtInformation, [mbOk], 0)
  else
  begin
    try
      try

        {Open}

        assignfile(hdx, HdxPath);
        reset(hdx);

        {Read file header}

        s := '';
        repeat
          read(hdx, c);
          if not (c in [#10,#26]) then
            s := s + c;
        until c = #26;

        {Check we've got a HDX file header}

        if s <> 'Borland Help Data File' then
        begin
          fmtStr(msg, 'ExtraHelp: Invalid HDX header "%s"', [s]);
          MessageDlg(msg, mtInformation, [mbOk], 0)
        end
        else
        begin

          {First byte after header contains number of entries}

          read(hdx, c);
          count := ord(c);

          {Next are the actual help file names, preceded by length byte}

          for i := 1 to count do
          begin
            s := '';
            read(hdx,s[0]);
            for j := 1 to ord(s[0]) do
              read(hdx, s[j]);

            {Add HelpFile object to result list}

            HelpFile := THelpFile.Create(s, SearchPaths);
            list.addObject(helpFile.Title, HelpFile);

          end;

        end;
      except
        on E: Exception do
           HandleException(E);
      end; {try..except}
    finally
      closefile(hdx);
    end; {try..finally}
  end; {if file exists}
end;

{Returns string identifying VCS - whats it used for?}

function TEHClient.GetIDString: string;
begin
  try
    Result := 'Jons.ExtraHelp';
  except
    on E: Exception do
      HandleException(E);
  end;
end;

{Called when user chooses from menu}

procedure TEHClient.ExecuteVerb(Index: Integer);
begin
  try
    if index<MenuItems.count then
    begin
      if MenuItems.Objects[index] is THelpFile then
        OpenHelpFile(THelpFile(MenuItems.Objects[index]).Path)
      else if Index = MenuItems.Count-2 then   {Options dialog}
        Options
      else if Index = MenuItems.Count-1 then   {About dialog}
        About;
    end;
  except
    on E: Exception do
      HandleException(E);
  end;
end;

{Returns name of menu title to add to menu bar}

function TEHClient.GetMenuName: string;
begin
  try
    Result := 'Extr&aHelp';
  except
    on E: Exception do
      HandleException(E);
  end;
end;

{Returns name of menu item to add to menu}

function TEHClient.GetVerb(Index: Integer): string;
begin
  try
    if index<MenuItems.count then
      result := MenuItems[index]
    else
      Result := '';
  except
    on E: Exception do
      HandleException(E);
  end;
end;

{Returns the number of menu items in our menu}

function TEHClient.GetVerbCount: Integer;
begin
  try
    Result := MenuItems.count;
  except
    on E: Exception do
      HandleException(E);
  end;
end;

{Returns state of menu - enabled/disabled, checked/unchecked}

function TEHClient.GetVerbState(Index: Integer): Word;
begin
  result := 0;
  try
    if index<MenuItems.count then
    begin
      {If options/about, or help file is available: enable menu item}
      if (MenuItems.Objects[index]=nil) or THelpFile(MenuItems.Objects[Index]).Available then
        Result := vsEnabled;
    end;
  except
    on E: Exception do
      HandleException(E);
  end;
end;

{Called when user changes project - unused by ExtraHelp}

procedure TEHClient.ProjectChange;
begin
  {must be present to prevent call to abstract method of base class}
end;

procedure TEHClient.OpenHelpFile(HelpFile: String);
  var code: word;
      cmdLine,msg: string;
begin
  try
    cmdLine := 'WINHELP.EXE '+HelpFile+#0;
    code := WinExec(@cmdLine[1], sw_shownormal);

    msg := 'OK';
    case code of
      0:	msg := 'Out of memory, executable file corrupt, or relocations invalid.';
      2:	msg := 'File not found.';
      3:	msg := 'Path not found.';
      5:	msg := 'Attempt made to dynamically link to task, or sharing or network-protection error.';
      6:	msg := 'Library required separate data segments for each task.';
      8:	msg := 'Insufficient memory to start application.';
      10:	msg := 'Windows version incorrect.';
      11:	msg := 'Executable file invalid.';
      12:	msg := 'Application designed for a different operating system.';
      13:	msg := 'Application designed for MS-DOS 4.0.';
      14:	msg := 'Type of executable file unknown.';
      15:	msg := 'Attempt made to load real-mode application (developed for an earlier version of Windows).';
      16:	msg := 'Attempt made to load a second instance of executable file.';
      19:	msg := 'Attempt made to load a compressed executable file.';
      20:	msg := 'Dynamic-link library (DLL) file invalid.';
      21:	msg := 'Application requires 32-bit extensions.';
    end;

    if msg <> 'OK' then
      MessageDlg('Error running WinHelp:'+#13#10+msg,mtInformation,[mbOk], 0);

  except
    on E: Exception do
      HandleException(e);
  end;
end;

procedure TEHClient.About;
  var dlg: TAboutBox;
begin
  try
    try
      dlg := TAboutBox.Create(nil);
      dlg.showModal;
    finally
      dlg.free;
    end;
  except
    on E: Exception do
      HandleException(e);
  end;
end;

procedure TEHClient.Options;
  var dlg: TOptionsDlg;
begin
  try
    try
      dlg := TOptionsDlg.Create(nil);
      dlg.showModal;
      showMessage('You must restart Delphi for options to take effect');
    finally
      dlg.free;
    end;
  except
    on E: Exception do
      HandleException(e);
  end;
end;

end.

